/*
 * VEML6075.cpp
 *
 * Arduino library for the Vishay VEML6075 UVA/UVB i2c sensor.
 *
 * Author: Sean Caulfield <sean@yak.net>
 * 
 * License: GPLv2.0
 *
 */

/*name=VEML6075 Library
version=1.0.0
author=Sean Caulfield <sean@yak.net>
maintainer=Sean Caulfield <sean@yak.net>
sentence=Library for the VEML6075 UVA/UVB Sensor
paragraph=Library for Vishay's VEML6075 UVA/UVB i2c Sensor
category=Sensors
url=https://github.com/schizobovine/VEML6075/
architectures=*
*/

#define VEML6075_ADDR  0x10
#define VEML6075_DEVID 0x26

// Reading the application note on calculation of UV index, the "dummy" channel
// value is actually not a dummy value at all, but the dark current count.
// NAMES ARE IMPORTANT PEOPLE.

#define VEML6075_REG_CONF        (0x00) // Configuration register (options below)
#define VEML6075_REG_UVA         (0x07) // UVA register
#define VEML6075_REG_DUMMY       (0x08) // Dark current register (NOT DUMMY)
#define VEML6075_REG_UVB         (0x09) // UVB register
#define VEML6075_REG_UVCOMP1     (0x0A) // Visible compensation register
#define VEML6075_REG_UVCOMP2     (0x0B) // IR compensation register
#define VEML6075_REG_DEVID       (0x0C) // Device ID register

#define VEML6075_CONF_IT_50MS    (0x00) // Integration time = 50ms (default)
#define VEML6075_CONF_IT_100MS   (0x10) // Integration time = 100ms
#define VEML6075_CONF_IT_200MS   (0x20) // Integration time = 200ms
#define VEML6075_CONF_IT_400MS   (0x30) // Integration time = 400ms
#define VEML6075_CONF_IT_800MS   (0x40) // Integration time = 800ms
#define VEML6075_CONF_IT_MASK    (0x8F) // Mask off other config bits

#define VEML6075_CONF_HD_NORM    (0x00) // Normal dynamic seetting (default)
#define VEML6075_CONF_HD_HIGH    (0x08) // High dynamic seetting

#define VEML6075_CONF_TRIG       (0x04) // Trigger measurement, clears by itself

#define VEML6075_CONF_AF_OFF     (0x00) // Active force mode disabled (default)
#define VEML6075_CONF_AF_ON      (0x02) // Active force mode enabled (?)

#define VEML6075_CONF_SD_OFF     (0x00) // Power up
#define VEML6075_CONF_SD_ON      (0x01) // Power down

// To calculate the UV Index, a bunch of empirical/magical coefficients need to
// be applied to UVA and UVB readings to get a proper composite index value.
// Seems pretty hand wavey, though not nearly as annoying as the dark current
// not being subtracted out by default.

#define VEML6075_UVI_UVA_VIS_COEFF (2.22) // aka coeff "A"
#define VEML6075_UVI_UVA_IR_COEFF  (1.33) // aka coeff "B"
#define VEML6075_UVI_UVB_VIS_COEFF (2.95) // aka coeff "C"
#define VEML6075_UVI_UVB_IR_COEFF  (1.74) // aka coeff "D"

// Once the above offsets and crunching is done, there's a last weighting
// function to convert the ADC counts into the UV index values. This handles
// both the conversion into irradiance (W/m^2) and the skin erythema weighting
// by wavelength--UVB is way more dangerous than UVA, due to shorter
// wavelengths and thus more energy per photon. These values convert the
// compensated values.
//
// NB These are the "open air" values given in the application note for the
// VEML6075.

#define VEML6075_UVI_UVA_RESPONSE (0.001461)
#define VEML6075_UVI_UVB_RESPONSE (0.002591)

enum veml6075_int_time {
  VEML6075_IT_50MS,
  VEML6075_IT_100MS,
  VEML6075_IT_200MS,
  VEML6075_IT_400MS,
  VEML6075_IT_800MS
};
typedef enum veml6075_int_time veml6075_int_time_t;

void begin6075()
{
  
      uint8_t config;
      config = 0;
      config |= VEML6075_CONF_SD_OFF;

  // App note only provided math for this one, so be advised that changing it
  // will give you "undefined" results from all the calculations.
  // Might be able to do a linear compensation for the integration time length?
      config |= VEML6075_CONF_IT_100MS;

      write16(VEML6075_REG_CONF, config); // do not work after poweron without configuration because in "switched off state"

}


void get_uvi() {

  Serial.println("Scanning...");

      uint8_t config;
      config = 0;
      config |= VEML6075_CONF_SD_OFF;

  // App note only provided math for this one, so be advised that changing it
  // will give you "undefined" results from all the calculations.
  // Might be able to do a linear compensation for the integration time length?
      config |= VEML6075_CONF_IT_100MS;

      write16(VEML6075_REG_CONF, config); // do not work after poweron without configuration

      uint16_t did;

      did=getDevID();
      Serial.println(did, HEX);

    raw_uva = read16(VEML6075_REG_UVA);
    raw_uvb = read16(VEML6075_REG_UVB);
    raw_dark = read16(VEML6075_REG_DUMMY);
    raw_vis = read16(VEML6075_REG_UVCOMP1);
    raw_ir = read16(VEML6075_REG_UVCOMP2);

      Serial.println(raw_uva, HEX);
      Serial.println(raw_uvb, HEX);
      Serial.println(raw_dark, HEX);
      Serial.println(raw_vis, HEX);
      Serial.println(raw_ir, HEX);

      float fuva = getUVA();
      float fuvb = getUVB();
      float UVI = getUVIndex();

      Serial.print("UVA: ");
      Serial.println(fuva);
      Serial.print("UVB: ");
      Serial.println(fuvb);
      Serial.print("UVI: ");
      Serial.println(UVI);

      Serial.print("Lux: ");
      Serial.println(get_lux());
  


  delay(500);           // wait 5 seconds for next scan
}


uint16_t read16(uint8_t reg) {
  uint8_t msb = 0;
  uint8_t lsb = 0;

  Wire.beginTransmission(0x10);
  Wire.write(reg);
  Wire.endTransmission(false);

  Wire.requestFrom(0x10, 2);
  lsb = Wire.read();
  msb = Wire.read();

  return (msb << 8) | lsb;
}

void write16(uint8_t reg, uint16_t data) {
  Wire.beginTransmission(0x10);
  Wire.write(reg);
  Wire.write((uint8_t)(0xFF & (data >> 0))); // LSB
  Wire.write((uint8_t)(0xFF & (data >> 8))); // MSB
  Wire.endTransmission();
}

uint16_t getDevID() {
  return read16(VEML6075_REG_DEVID);
}

float getUVA() {
  float comp_vis;
  float comp_ir;
  float comp_uva;

  // Constrain compensation channels to positive values
  comp_ir  = max(raw_ir - raw_dark, 0);
  comp_vis = max(raw_vis - raw_dark, 0);
  comp_uva = max(raw_uva - raw_dark, 0);

  // Scale by coeffs from datasheet
  comp_vis *= VEML6075_UVI_UVA_VIS_COEFF;
  comp_ir  *= VEML6075_UVI_UVA_IR_COEFF;

  // Subtract out visible and IR components
  comp_uva = max(comp_uva - comp_ir, 0.0F);
  comp_uva = max(comp_uva - comp_vis, 0.0F);

  return comp_uva;
}

float getUVB() {
  float comp_vis;
  float comp_ir;
  float comp_uvb;

  // Constrain compensation channels to positive values
  comp_ir  = max(raw_ir - raw_dark, 0);
  comp_vis = max(raw_vis - raw_dark, 0);
  comp_uvb = max(raw_uvb - raw_dark, 0);

  // Scale by coeffs from datasheet
  comp_vis *= VEML6075_UVI_UVB_VIS_COEFF;
  comp_ir  *= VEML6075_UVI_UVB_IR_COEFF;

  // Subtract out visible and IR components
  comp_uvb = max(comp_uvb - comp_ir, 0.0F);
  comp_uvb = max(comp_uvb - comp_vis, 0.0F);

  return comp_uvb;
}

float getUVIndex() {
  float uva_weighted = getUVA() * VEML6075_UVI_UVA_RESPONSE;
  float uvb_weighted = getUVB() * VEML6075_UVI_UVB_RESPONSE;
  return (uva_weighted + uvb_weighted) / 2.0;
}

void get_uv_data()
{
    raw_uva = read16(VEML6075_REG_UVA);
    raw_uvb = read16(VEML6075_REG_UVB);
    raw_dark = read16(VEML6075_REG_DUMMY);
    raw_vis = read16(VEML6075_REG_UVCOMP1);
    raw_ir = read16(VEML6075_REG_UVCOMP2);

}
